#include <DHT.h>
#include <DHT_U.h>
#include <LiquidCrystal_I2C.h>
#include <SoftwareSerial.h>

#define BUTTON_PRESS_TIME 1000
#define DISPLAY_MESSAGE 5000
#define DHT_INTERVAL 900000
// #define GAS_INTERVAL 60000
#define GAS_INTERVAL 6000
#define GAS_THRESHOLD 400
// #define WATER_INTERVAL 300000
#define WATER_INTERVAL 3000
#define WATER_THRESHOLD 500
#define SWITCH_INTERVAL 10000
#define BUZZER_FREQ 1000

// PIN CONFIG
#define dht_pin A0
#define gas_pin A1
#define water_pin A2
#define dht_button_pin 2
#define gas_button_pin 3
#define water_button_pin 4
#define buzzer_pin 9
#define water_led_pin 12
#define gas_led_pin 13

bool sensor_changed = false;
bool gas_status = true;
bool gas_detected = false;
bool gas_alert = false;
bool water_status = true;
bool water_detected = false;
bool water_alert = false;
bool water_message = false;
bool gas_message = false;
int humidity;
int gas_level;
int water_level;
float temp;
String phoneNumber = "+40775268088";
unsigned long sensor_changed_time;
unsigned long dht_time = 0;
unsigned long gas_time = 0;
unsigned long gas_button_time = 0;
unsigned long water_button_time = 0;
unsigned long water_time = 0; 
unsigned long two_alerts_time = 0;

// DHT Sensor
DHT dht(dht_pin, DHT11);
void dht_func() {
  temp = dht.readTemperature();
  Serial.print("Temperatura: ");
  Serial.print(temp);
  Serial.println("°C");
  humidity = dht.readHumidity();
  Serial.print("Umiditate: ");
  Serial.print(humidity);
  Serial.println("%");
  dht_time = millis();
}

// LCD
LiquidCrystal_I2C lcd(0x27, 16, 2);
void lcd_func() {
  lcd.setCursor(0, 0);
  lcd.print("Temp: ");
  lcd.print(temp);
  lcd.print("C");
  
  if (!sensor_changed && !gas_alert && !water_alert) {
    lcd.setCursor(0,1);
    lcd.print("Humidity: ");
    lcd.print(humidity);
    lcd.print("%   ");
  }
}

// Gas sensor
void gas_func() {
  gas_time = millis();
  gas_level = analogRead(gas_pin);
  Serial.print("Gas: ");
  Serial.println(gas_level);
  if (!gas_detected) {
    if (gas_level < GAS_THRESHOLD) {
      gas_detected = false;
      gas_alert = false;
      gas_message = false;
    }
    else
      gas_detected = true;
  } else {
    if (gas_level < GAS_THRESHOLD)
      gas_detected = false;
    else {
      gas_detected = true;
      gas_alert = true;
    }
  }
}

// Water sensor
void water_func() {
  water_time = millis();
  water_level = analogRead(water_pin);
  Serial.print("Water: ");
  Serial.println(water_level);
  if (!water_detected) {
    if (water_level < WATER_THRESHOLD) {
      water_detected = false;
      water_alert = false;
      water_message = false;
    }
    else
      water_detected = true;
  } else { 
    if (water_level < WATER_THRESHOLD)
      water_detected = false;
    else {
      water_detected = true;
      water_alert = true;
    }
  }
  if (gas_alert)
    two_alerts_time = millis();
}

void buzzer_func() {
  if (gas_alert || water_alert)
    if ((millis() / 3000) % 2 == 0)
      tone(buzzer_pin, 500);
    else 
      noTone(buzzer_pin);
  else
    noTone(buzzer_pin);
}

SoftwareSerial sim(10, 11);

bool pressed(int pin, unsigned long time) {
  return digitalRead(pin) == LOW && millis() - time > BUTTON_PRESS_TIME;
}

void set_led(int pin, bool status) {
  if (status)
    digitalWrite(pin, HIGH);
  else
    digitalWrite(pin, LOW);
}

void sendMessage(String SMS){
  sim.println("AT+CMGF=1");    //Sets the GSM Module in Text Mode
  delay(200);
  sim.println("AT+CMGS=\"" + phoneNumber + "\"\r");
  delay(200);
  sim.println(SMS);
  delay(100);
  sim.println((char)26);// ASCII code of CTRL+Z
}

void setup() {
  Serial.begin(9600);
  dht.begin();
  dht_func();
  lcd.init();
  lcd.backlight();
  gas_func();
  water_func();
  sim.begin(9600);

  pinMode(dht_button_pin, INPUT_PULLUP);
  pinMode(gas_button_pin, INPUT_PULLUP);
  pinMode(water_button_pin, INPUT_PULLUP);
  pinMode(water_led_pin, OUTPUT);
  pinMode(gas_led_pin, OUTPUT);
  pinMode(buzzer_pin, OUTPUT);
}

void loop() {
  if (millis() - dht_time > DHT_INTERVAL)
    dht_func();
  if (millis() - gas_time > GAS_INTERVAL && gas_status)
    gas_func();
  if (millis() - water_time > WATER_INTERVAL && water_status)
    water_func();
  lcd_func();

  if (pressed(dht_button_pin, dht_time))
    dht_func();

  set_led(gas_led_pin, !gas_status);
  set_led(water_led_pin, !water_status);

  if(pressed(gas_button_pin, gas_button_time)) {
    gas_button_time = millis();
    gas_status = !gas_status;
    // Display the status change
    if (!gas_status) {
      lcd.setCursor(0,1);
      lcd.print("Gas sensor off  ");
      sensor_changed_time = millis();
      gas_alert = false;
      gas_detected = false;
    } else {
      lcd.setCursor(0,1);
      lcd.print("Gas sensor on   ");
      sensor_changed_time = millis();
      gas_func();
    }
  }

  if(pressed(water_button_pin, water_button_time)) {
    water_button_time = millis();
    water_status = !water_status;
    if(!water_status) {
      lcd.setCursor(0,1);
      lcd.print("Water sensor off");
      sensor_changed_time = millis();
      water_alert = false;
      water_detected = false;
    } else{
      lcd.setCursor(0,1);
      lcd.print("Water sensor on ");
      sensor_changed_time = millis();
      water_func();
    }
  }

  if (gas_alert && water_alert) {
    if (!gas_message || !water_message){
      sendMessage("GAS AND WATER ALERT");
      gas_message = true;
      water_message = true;
    }
    if (((millis() - two_alerts_time) / SWITCH_INTERVAL) %2 == 0) {
      lcd.setCursor(0,1);
      lcd.print("Gas alert       ");
    } else {
      lcd.setCursor(0,1);
      lcd.print("Water alert     ");
    }
  } else{
    if (gas_alert) {
      lcd.setCursor(0,1);
      lcd.print("Gas alert       ");
      if (!gas_message) {
        sendMessage("GAS ALERT");
        gas_message = true;
      }
    }

    if (water_alert) {
      lcd.setCursor(0,1);
      lcd.print("Water alert     ");
      if (!water_message) {
        sendMessage("WATER ALERT");
        water_message = true;
      }
    }
  }

  if (millis() - sensor_changed_time > DISPLAY_MESSAGE)
    sensor_changed = false;
  else 
    sensor_changed = true;

  buzzer_func();
}
